@@ -26,6 +26,7 @@
 #include <glib.h>
 #include <glib/gprintf.h>
 #include <gtk/gtk.h>
+#include "actions.h"
 #include "eggsmclient.h"
 #include "goo-player-info.h"
 #include "goo-stock.h"
@@ -52,7 +53,6 @@ gboolean                   notification_supports_actions = FALSE;
 /* -- command line arguments -- */
 
 
-GtkApplication        *main_application = NULL;
 int                    arg_auto_play = FALSE;
 int                    arg_toggle_visibility = FALSE;
 static int             arg_toggle_play = FALSE;
@@ -60,8 +60,6 @@ static int             arg_stop = FALSE;
 static int             arg_next = FALSE;
 static int             arg_prev = FALSE;
 static int             arg_eject = FALSE;
-static int             arg_volume_up = FALSE;
-static int             arg_volume_down = FALSE;
 static int             arg_quit = FALSE;
 static gboolean        arg_version = FALSE;
 static char           *arg_device = NULL;
@@ -93,12 +91,6 @@ static const GOptionEntry options[] = {
 	{ "toggle-visibility", '\0', 0, G_OPTION_ARG_NONE, &arg_toggle_visibility,
           N_("Toggle the main window visibility"),
           0 },
-	{ "volume-up", '\0', 0, G_OPTION_ARG_NONE, &arg_volume_up,
-          N_("Volume Up"),
-          0 },
-	{ "volume-down", '\0', 0, G_OPTION_ARG_NONE, &arg_volume_down,
-          N_("Volume Down"),
-          0 },
 	{ "quit", '\0', 0, G_OPTION_ARG_NONE, &arg_quit,
           N_("Quit the application"),
           0 },
@@ -217,7 +209,15 @@ goo_restore_session (EggSMClient  *client,
 /* -- main application -- */
 
 
-typedef GtkApplication      GooApplication;
+GtkApplication *Main_Application = NULL;
+
+
+typedef struct {
+	GtkApplication __parent;
+	GSettings *settings;
+} GooApplication;
+
+
 typedef GtkApplicationClass GooApplicationClass;
 
 
@@ -227,14 +227,19 @@ G_DEFINE_TYPE (GooApplication, goo_application, GTK_TYPE_APPLICATION)
 static void
 goo_application_finalize (GObject *object)
 {
+	GooApplication *self = (GooApplication *) object;
+
+	g_object_unref (self->settings);
 	G_OBJECT_CLASS (goo_application_parent_class)->finalize (object);
 }
 
 
 static void
-goo_application_init (GooApplication *app)
+goo_application_init (GooApplication *self)
 {
 	g_set_application_name (_("CD Player"));
+
+	self->settings = g_settings_new (GOOBOX_SCHEMA_PLAYLIST);
 }
 
 
@@ -244,7 +249,6 @@ goo_application_activate (GApplication *application)
 	GtkWidget *window;
 
 	window = goo_window_new (NULL);
-	gtk_window_set_application (GTK_WINDOW (window), GTK_APPLICATION (application));
 	gtk_widget_show (window);
 }
 
@@ -372,18 +376,6 @@ goo_application_command_line (GApplication            *application,
 	else if (arg_toggle_visibility) {
 		goo_window_toggle_visibility (GOO_WINDOW (window));
 	}
-	else if (arg_volume_up) {
-		double volume;
-
-		volume = goo_window_get_volume (GOO_WINDOW (window));
-		goo_window_set_volume (GOO_WINDOW (window), volume + VOLUME_STEP);
-	}
-	else if (arg_volume_down) {
-		double volume;
-
-		volume = goo_window_get_volume (GOO_WINDOW (window));
-		goo_window_set_volume (GOO_WINDOW (window), volume - VOLUME_STEP);
-	}
 	else if (arg_quit) {
 		goo_window_close (GOO_WINDOW (window));
 	}
@@ -414,8 +406,6 @@ goo_application_command_line (GApplication            *application,
 	arg_prev = FALSE;
 	arg_eject = FALSE;
 	arg_toggle_visibility = FALSE;
-	arg_volume_up = FALSE;
-	arg_volume_down = FALSE;
 	arg_quit = FALSE;
 	g_free (arg_device);
 	arg_device = NULL;
@@ -465,6 +455,230 @@ goo_application_local_command_line (GApplication   *application,
 }
 
 
+/* -- application menu  -- */
+
+
+static void
+toggle_action_activated (GSimpleAction *action,
+                         GVariant      *parameter,
+                         gpointer       data)
+{
+	GVariant *state;
+
+	state = g_action_get_state (G_ACTION (action));
+	g_action_change_state (G_ACTION (action), g_variant_new_boolean (! g_variant_get_boolean (state)));
+
+	g_variant_unref (state);
+}
+
+
+static void
+update_app_menu_sensitivity (GooApplication *application)
+{
+	GVariant *state;
+	gboolean  play_all;
+
+	state = g_action_get_state (g_action_map_lookup_action (G_ACTION_MAP (application), PREF_PLAYLIST_PLAYALL));
+	play_all = g_variant_get_boolean (state);
+	g_variant_unref (state);
+
+	g_simple_action_set_enabled (G_SIMPLE_ACTION (g_action_map_lookup_action (G_ACTION_MAP (application), PREF_PLAYLIST_REPEAT)), play_all);
+	g_simple_action_set_enabled (G_SIMPLE_ACTION (g_action_map_lookup_action (G_ACTION_MAP (application), PREF_PLAYLIST_SHUFFLE)), play_all);
+}
+
+
+static void
+activate_play_all (GSimpleAction *action,
+		   GVariant      *parameter,
+		   gpointer       user_data)
+{
+	GooApplication *application = user_data;
+
+	g_simple_action_set_state (action, parameter);
+	g_settings_set_boolean (application->settings, PREF_PLAYLIST_PLAYALL, g_variant_get_boolean (parameter));
+	update_app_menu_sensitivity (application);
+}
+
+
+static void
+activate_repeat (GSimpleAction *action,
+		 GVariant      *parameter,
+		 gpointer       user_data)
+{
+	GooApplication *application = user_data;
+
+	g_simple_action_set_state (action, parameter);
+	g_settings_set_boolean (application->settings, PREF_PLAYLIST_REPEAT, g_variant_get_boolean (parameter));
+	update_app_menu_sensitivity (application);
+}
+
+
+static void
+activate_shuffle (GSimpleAction *action,
+		  GVariant      *parameter,
+		  gpointer       user_data)
+{
+	GooApplication *application = user_data;
+
+	g_simple_action_set_state (action, parameter);
+	g_settings_set_boolean (application->settings, PREF_PLAYLIST_SHUFFLE, g_variant_get_boolean (parameter));
+	update_app_menu_sensitivity (application);
+}
+
+
+static void
+activate_preferences (GSimpleAction *action,
+		      GVariant      *parameter,
+		      gpointer       user_data)
+{
+	GApplication *application = user_data;
+	GList        *windows;
+
+	windows = gtk_application_get_windows (GTK_APPLICATION (application));
+	if (windows != NULL)
+		activate_action_preferences (NULL, windows->data);
+}
+
+
+static void
+activate_help (GSimpleAction *action,
+               GVariant      *parameter,
+               gpointer       user_data)
+{
+	GApplication *application = user_data;
+	GList        *windows;
+
+	windows = gtk_application_get_windows (GTK_APPLICATION (application));
+	if (windows != NULL)
+		activate_action_manual (NULL, windows->data);
+}
+
+
+static void
+activate_about (GSimpleAction *action,
+		GVariant      *parameter,
+		gpointer       user_data)
+{
+	GApplication *application = user_data;
+	GList        *windows;
+
+	windows = gtk_application_get_windows (GTK_APPLICATION (application));
+	if (windows != NULL)
+		activate_action_about (NULL, windows->data);
+}
+
+
+static void
+activate_quit (GSimpleAction *action,
+               GVariant      *parameter,
+               gpointer       user_data)
+{
+	GApplication *application = user_data;
+	GList        *windows;
+
+	windows = gtk_application_get_windows (GTK_APPLICATION (application));
+	if (windows != NULL)
+		activate_action_quit (NULL, windows->data);
+}
+
+
+static void
+pref_playlist_playall_changed (GSettings  *settings,
+	  	 	       const char *key,
+	  	 	       gpointer    user_data)
+{
+	GooApplication *application = user_data;
+
+	g_simple_action_set_state (G_SIMPLE_ACTION (g_action_map_lookup_action (G_ACTION_MAP (application), PREF_PLAYLIST_PLAYALL)),
+				   g_variant_new_boolean (g_settings_get_boolean (application->settings, PREF_PLAYLIST_PLAYALL)));
+	update_app_menu_sensitivity (application);
+}
+
+
+static void
+pref_playlist_repeat_changed (GSettings  *settings,
+	  	 	      const char *key,
+	  	 	      gpointer    user_data)
+{
+	GooApplication *application = user_data;
+
+	g_simple_action_set_state (G_SIMPLE_ACTION (g_action_map_lookup_action (G_ACTION_MAP (application), PREF_PLAYLIST_REPEAT)),
+				   g_variant_new_boolean (g_settings_get_boolean (application->settings, PREF_PLAYLIST_REPEAT)));
+	update_app_menu_sensitivity (application);
+}
+
+
+static void
+pref_playlist_shuffle_changed (GSettings  *settings,
+	  	 	       const char *key,
+	  	 	       gpointer    user_data)
+{
+	GooApplication *application = user_data;
+
+	g_simple_action_set_state (G_SIMPLE_ACTION (g_action_map_lookup_action (G_ACTION_MAP (application), PREF_PLAYLIST_SHUFFLE)),
+				   g_variant_new_boolean (g_settings_get_boolean (application->settings, PREF_PLAYLIST_SHUFFLE)));
+	update_app_menu_sensitivity (application);
+}
+
+
+static const GActionEntry app_menu_entries[] = {
+	{ "preferences",  activate_preferences },
+	{ PREF_PLAYLIST_PLAYALL, toggle_action_activated, NULL, "true", activate_play_all },
+	{ PREF_PLAYLIST_REPEAT, toggle_action_activated, NULL, "false", activate_repeat },
+	{ PREF_PLAYLIST_SHUFFLE, toggle_action_activated, NULL, "true", activate_shuffle },
+	{ "help",  activate_help },
+	{ "about", activate_about },
+	{ "quit",  activate_quit }
+};
+
+
+static void
+initialize_app_menu (GApplication *application)
+{
+	GooApplication *self = (GooApplication *) application;
+	GtkBuilder     *builder;
+
+	g_action_map_add_action_entries (G_ACTION_MAP (application),
+					 app_menu_entries,
+					 G_N_ELEMENTS (app_menu_entries),
+					 application);
+
+	builder = _gtk_builder_new_from_resource ("app-menu.ui");
+	gtk_application_set_app_menu (GTK_APPLICATION (application),
+				      G_MENU_MODEL (gtk_builder_get_object (builder, "app-menu")));
+
+	g_simple_action_set_state (G_SIMPLE_ACTION (g_action_map_lookup_action (G_ACTION_MAP (application), PREF_PLAYLIST_PLAYALL)),
+				   g_variant_new_boolean (g_settings_get_boolean (self->settings, PREF_PLAYLIST_PLAYALL)));
+	g_simple_action_set_state (G_SIMPLE_ACTION (g_action_map_lookup_action (G_ACTION_MAP (application), PREF_PLAYLIST_REPEAT)),
+				   g_variant_new_boolean (g_settings_get_boolean (self->settings, PREF_PLAYLIST_REPEAT)));
+	g_simple_action_set_state (G_SIMPLE_ACTION (g_action_map_lookup_action (G_ACTION_MAP (application), PREF_PLAYLIST_SHUFFLE)),
+				   g_variant_new_boolean (g_settings_get_boolean (self->settings, PREF_PLAYLIST_SHUFFLE)));
+
+	g_signal_connect (self->settings,
+			  "changed::" PREF_PLAYLIST_PLAYALL,
+			  G_CALLBACK (pref_playlist_playall_changed),
+			  self);
+	g_signal_connect (self->settings,
+			  "changed::" PREF_PLAYLIST_SHUFFLE,
+			  G_CALLBACK (pref_playlist_shuffle_changed),
+			  self);
+	g_signal_connect (self->settings,
+			  "changed::" PREF_PLAYLIST_REPEAT,
+			  G_CALLBACK (pref_playlist_repeat_changed),
+			  self);
+
+	g_object_unref (builder);
+}
+
+
+static void
+goo_application_startup (GApplication *application)
+{
+	G_APPLICATION_CLASS (goo_application_parent_class)->startup (application);
+	initialize_app_menu (application);
+}
+
+
 static void
 goo_application_class_init (GooApplicationClass *klass)
 {
@@ -478,6 +692,7 @@ goo_application_class_init (GooApplicationClass *klass)
 	application_class->activate = goo_application_activate;
 	application_class->command_line = goo_application_command_line;
 	application_class->local_command_line = goo_application_local_command_line;
+	application_class->startup = goo_application_startup;
 }
 
 
@@ -501,11 +716,6 @@ main (int argc, char *argv[])
 
 	program_argv0 = argv[0];
 
-	if (! g_thread_supported ()) {
-		g_thread_init (NULL);
-		gdk_threads_init ();
-	}
-
 	/* text domain */
 
 	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
@@ -514,10 +724,9 @@ main (int argc, char *argv[])
 
 	/* run the main application */
 
-	application = main_application = goo_application_new ();
-	gdk_threads_enter ();
+	application = Main_Application = goo_application_new ();
 	status = g_application_run (G_APPLICATION (application), argc, argv);
-	gdk_threads_leave ();
+
 	g_object_unref (application);
 
 	return status;
@@ -535,7 +744,7 @@ main_get_window_from_device (const char *device)
 	if (device == NULL)
 		return NULL;
 
-	for (scan = gtk_application_get_windows (main_application); scan; scan = scan->next) {
+	for (scan = gtk_application_get_windows (GTK_APPLICATION (g_application_get_default ())); scan; scan = scan->next) {
 		GooWindow *window = scan->data;
 
 		if (g_strcmp0 (goo_player_get_device (goo_window_get_player (window)), device) == 0)
@@ -658,7 +867,7 @@ notification_has_persistence (void)
 
 #else
 
-	return false;
+	return FALSE;
 
 #endif /* ENABLE_NOTIFICATION */
 }
@@ -729,15 +938,15 @@ system_notify (GooWindow       *window,
 						window,
 						NULL);
 
-		notify_notification_set_hint_byte (notification,
-						   "action-icons",
-						   TRUE);
+		notify_notification_set_hint (notification,
+					      "action-icons",
+					      g_variant_new_boolean (TRUE));
 	}
 
 	if (notification_supports_persistence)
-		notify_notification_set_hint_byte (notification,
-						    "resident" /* "transient" */,
-						    TRUE);
+		notify_notification_set_hint (notification,
+					      "resident" /* "transient" */,
+					      g_variant_new_boolean (TRUE));
 
 	notify_notification_show (notification, NULL);
 
